#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "tpw.h"
#include "char.h"
#include "party.h"
#include "rename.h"

unsigned long promptLong(char *say, unsigned long ldefault) {
	unsigned long n;
	printf("%s : ", say);
	if (scanf(" %lu", &n)) {
		return n;
	}
	return ldefault;
}

int editCharacter(char * filename, int exists) {
FILE *fp;
struct TPW tpw;
struct PARTY party;
struct BARDCHAR bard;
int i, done;
char buffer[255];
char c;

if (exists) {
fp = fopen(filename, "rb");
if (!fp) return 1; /* unable to open file for read */
fgetTPW(&tpw, fp);
if (isparty(tpw)) {
	getparty(&party, fp);
} else {
	fgetBardCharacter(&bard, fp);
}
fclose(fp);
} else { /* if it is new we need some itialization */
strcpy(tpw.name,"NAMELESS"); /* set default char name  */
tpw.party          = 1; /* 1 is character, 2 is party  */
bard.level         = 1; /* Must be at least level 1    */
bard.hp            = 6; /* I don't recall seeing less  */
bard.mstrength     = 3; /* The genre of game uses dice */
bard.miq           = 3; /* max stat i've come across   */
bard.mdexterity    = 3; /* has been 18.  Assuming 3    */
bard.mconstitution = 3; /* dice are used for stats.    */
bard.mluck         = 3;
bard.mlevel        = 1;
} /* if exists else */ 

/* We are only editing characters now, not parties     */
if (isparty(tpw)) return EXIT_FAILURE;
done=0;
do {
printf("Character name is currently %s\n",tpw.name);
shortDumpChar(&bard);
printf(
"\nSelect on of the following:"
"\n  A) Cure All"
"\n  R) Rename Character"
"\n  W) Change Race"
"\n  C) Change Class"
"\n  L) Change Level"
"\n  P) Change Hit Points"
"\n  S) Change Stats"
"\n  H) Change Status"
"\n  E) Change Experience"
"\n  G) Change Gold"
"\n  I) Edit Items"
"\n  X) Done"
"\n> ");
scanf(" %c", &c);
switch (c) {
case '#':
case '!':
case '/':
case ';':
case ':':
case '(':
case '[':
case '\'':
/* Throw away line for comments */
	fgets(buffer,255,stdin);
	break;
case 'a':
case 'A':
/* Cure all */
	bard.status = 0;
	done = 1;
	break;
case 'r':
case 'R':
/* Edit name */
	printf("Type new name for charcter.> ");
	scanf(" %[A-Za-z ]",buffer);
	strncpy(tpw.name, buffer, 14);
	tpw.name[14]='\0';
	for (i=0; i<14; i++) tpw.name[i] = toupper(tpw.name[i]);
	printf("New name is %s\n", tpw.name);
	break;
case 'h':
case 'H':
/* Edit Status */
	bard.status    = promptStatus(bard.status);
	break;
case 'w':
case 'W':
/* Edit Race */
	bard.race      = promptRace(Human);
	break;
case 'c':
case 'C':
/* Edit Class */
	bard.class     = promptClass(Warrior);
	/* Only mages need spell points and spell levels, others will be 0. */
	/* Really should be at least 1 in spell level for current class */
	if (isMage(bard.class)) {
	bard.msppt         = prompt("Spell Points", 0, 32767,
			bard.iq+bard.luck);
	printf("\nSpell Levels:\n");
	bard.conjlevel = prompt("Conjurer", 0, 7, bard.class==Conjurer?1:0);
	bard.magilevel = prompt("Magician", 0, 7, bard.class==Magician?1:0);
	bard.sorclevel = prompt("Sorceror", 0, 7, bard.class==Sorceror?1:0);
	bard.wizdlevel = prompt("Wizard  ", 0, 7, bard.class==Wizard  ?1:0);
	} else {
	/* Non-mages should have no spell points or levels */
	bard.msppt     = 0;
	bard.conjlevel = 0;
	bard.magilevel = 0;
	bard.sorclevel = 0;
	bard.wizdlevel = 0;
	}
	bard.attacks = getsExtraAttacks(bard.class)?
		prompt("Extra attacks", 0, 15, 0):0;
	break;
case 'l':
case 'L':
/* Edit Level */
	bard.level     = prompt("Level       ", 1, 255, 1);
	break;
case 'e':
case 'E':
/* Edit Experience */
	bard.experience= promptLong("Experience  ", 0);
	break;
case 'g':
case 'G':
/* Edit Gold */
	bard.gold      = promptLong("Gold        ", 0);
	break;
case 'p':
case 'P':
	bard.hp        = prompt("Hit Points  ", 1, 32767, 6);
	break;
case 's':
case 'S':
/* Edit Stats */
	bard.mstrength     = prompt("Strength    ", 3, 255, 3);
	bard.miq           = prompt("IQ          ", 3, 255, 3);
	bard.mdexterity    = prompt("Dexterity   ", 3, 255, 3);
	bard.mconstitution = prompt("Constitution", 3, 255, 3);
	bard.mluck         = prompt("Luck        ", 3, 255, 3);
	break;
case 'i':
case 'I':
/* Edit Items */
	editItems(&bard);
	break;
case 'x':
case 'X':
/* Done editting */
	done=1;
	break;
}
} while (!done);
/* Let's max out stats that were reduced due to curse or illness */
bard.condition     = bard.hp;
bard.strength      = bard.mstrength;
bard.iq            = bard.miq;
bard.dexterity     = bard.mdexterity;
bard.constitution  = bard.mconstitution;
bard.luck          = bard.mluck;
bard.sppt          = bard.msppt;
bard.mlevel        = bard.level;
/* Bards get 1 song per level, other classes should have no songs. */
bard.songs         = isBard(bard.class)?bard.level:0;
printf("%-14s %-12s ", tpw.name, filename);
dumpcharacter(&bard);
printf("Save changes? (Y/N) ");
scanf(" %s", buffer);
buffer[0] = toupper(buffer[0]);
if (buffer[0]!='Y') return 4; /* user aborted change */
fp = fopen(filename, "wb");
if (!fp) return 2; /* unable to open file for write */
fputTPW(&tpw, fp);
if (isparty(tpw)) {
	putparty(&party, fp);
} else {
	fputBardCharacter(&bard, fp);
}

fclose(fp);
return 0;
}

int prompt(char *myprompt, int min, int max, int idefault) {
	int n;
	printf("%s (%d..%5d): ", myprompt, min, max);
	if (scanf("%d", &n)) {
		if ((n>=min)&&(n<=max)) return n;
	}
	return idefault;
}

int promptStatus(int status) {
int done=0;
int n, c;
n=status;
while (!done) {
printf("\nYour current status is %d.  : ", n);
printStatus(n);
printf(
"\nHow would you like to modify your status?"
"\n  0) Cure All"
"\n  1) Toggle Impairments"
"\n  2) Set"
"\n  3) Exit"
"\n> ");
scanf(" %d", &c);
switch (c) {
case 0:
	n=0;
	break;
case 1:
	n=promptYN("Toggle Death    ? ")?n^BT_DEAD:n;
	n=promptYN("Toggle Old      ? ")?n^BT_OLD:n;
	n=promptYN("Toggle Stoned   ? ")?n^BT_STONED:n;
	n=promptYN("Toggle Paralysis? ")?n^BT_PARALYZED:n;
	n=promptYN("Toggle Poisoned ? ")?n^BT_POISONED:n;
	n=promptYN("Toggle Nuts     ? ")?n^BT_NUTS:n;
	n=promptYN("Toggle Possessed? ")?n^BT_POSSESSED:n;
	break;
case 2:
	n=prompt("Value for Status", 0, 255, 0);
	break;
case 3:
	done=1;
	break;
} /* switch */
} /* while not done */
printf("\nYour old status was %d.  : ", status);
printStatus(status);
printf("\nYour current status is %d.  : ", n);
printStatus(n);
return promptYN("\nUpdate Status   ? ")?n:status;
}

int promptRace(int rdefault) {
	int i, n;
	
	printf("Choose your race:");
	for (i=Human; i<=Gnome; i++) { printf("\n %d) ",i); printRace(i); }
	printf("\n(0..%d): ",Gnome);
	if (scanf("%d", &n)) {
		if ((n>=Human)&&(n<=Gnome)) return n;
	}
	return rdefault;
}

int promptClass(int cdefault) {
	int i, n;
	
	printf("Choose your class:");
	for (i=Warrior; i<=Wizard; i++) { printf("\n %d) ",i); printClass(i); }
	printf("\n(0..%d): ",Wizard);
	if (scanf("%d", &n)) {
		if ((n>=Warrior)&&(n<=Wizard)) return n;
	}
	return cdefault;
}

int editItems(struct BARDCHAR *bard) {
int i;
for (;;) {
printf("\nItems:");
for (i=0; i<8; i++) {
  if (bard->item[i].code) {
        printf("\n%d) ",i+1);
        printItemStatus(bard->item[i].status);
        printItemDesc(bard->item[i].code);
  }
}
i=prompt("\nWhich item? (0 to end) ",0,8,0);
if (i==0) break;
showItemCodes();
printf("\n");
bard->item[i-1].code = 
	prompt("Code    : ", 0, 127, bard->item[i-1].code);
bard->item[i-1].status =
	prompt("Charges : ", 0, 63, 0);
if (!promptYN("Was the item identified? ")) 
	bard->item[i-1].status=bard->item[i-1].status|0x40;
if (promptYN("Is the item equiped? ")) 
	bard->item[i-1].status=bard->item[i-1].status|0x80;
} /* for */
return 0;
}

int promptYN(char *myprompt) {
char buffer[256];
printf("%s", myprompt);
scanf(" %s", buffer);
switch (buffer[0]) {
	case 'Y':
	case 'y':
		return 1;
		break;
}
return 0;
}

